--------------------------------------------------------------------------------------------------
-- script : sync-nas.lua
-- author : Dan Gill
-- September 2018
-- version: 1.1
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
-- desc   : The intention of this script is sync NAS AO Profiles and PPRs. This aids in the use of
--          the HA probe. The folder "script_logs" *MUST* be manually created under the nas
--          directory prior to running this script.
---------------------------------------------------------------------------------------------------        
---------------------------------------------------------------------------------------------------
-- Version  | Details
---------------------------------------------------------------------------------------------------
-- 1.0      | Initial Version
-- 1.1      | 9/20/2018 - Replace dofile with require and added package.path update; this makes
--            passing the path to the includes files simpler. Swapped out file.write and print
--            statements with output function from logging_functions.
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- Set Variables
-----------------------------------------------------------------------
-----------------------------------------------------------------------

package.path = package.path .. ";./scripts/includes/?.lua" -- Where are your require files?

local secondary_nas = "/DOMAIN/HUB/ROBOT" -- The name of failover nas
local probe  = "nas" -- The probe
local output_location = 2 -- 1 = stdout; 2 = file; 3 = both

local str_beg, str_end = string.find (SCRIPT_NAME,".",1,true)
local fname = "./script_logs/" .. left (SCRIPT_NAME, str_beg-1) .. ".log"

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- DO NOT EDIT BELOW THIS LINE
-----------------------------------------------------------------------
-----------------------------------------------------------------------

require ("error_functions")
require ("logging_functions")
require ("table_functions")

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- Define functions
-----------------------------------------------------------------------
-----------------------------------------------------------------------

local function get_probe_config(controller, prid)
   local args = pds.create()
   pds.putString(args, "name", prid)
   local probe_config, rc = nimbus.request(controller, "probe_config_get", args)
   pds.delete(args)
   
   if rc == NIME_OK then
      return probe_config
   else
      return nil
   end
end

local function copy_probe_config(robot, prid, section_name, source_config)
   for section,_ in pairs(source_config) do
      for key,value in pairs(source_config[section]) do
         if string.match(section, section_name) then
            local args = pds.create()
            pds.putString(args, "name", prid)
            pds.putString(args, "section", section)
            pds.putString(args, "key", key)
            pds.putString(args, "value", value)
            local set_probe_config, rc = nimbus.request(robot, "probe_config_set", args)
            pds.delete(args)
            
            output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Section: "..section.." Key: "..key.." Value: "..value.." written to "..robot.."/"..prid, output_location)
            
         end
      end
   end
end

local function main()
   if output_location == 2 or output_location == 3 then
      file.create (fname)
   end

   local host, rc = nimbus.request("controller", "get_info")
   local primary_nas_config = get_probe_config("controller", probe)
   
   if primary_nas_config ~= nil then 
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Primary NAS is /" .. host.domain .. "/" .. host.hubname .. "/" .. host.robotname, output_location)
      
      copy_probe_config(secondary_nas, probe, "/auto_operator/definitions", primary_nas_config)
      copy_probe_config(secondary_nas, probe, "/filters/exclude", primary_nas_config)
      copy_probe_config(secondary_nas, probe, "/filters/invisible", primary_nas_config)
      
      local restart,rc = nimbus.request(secondary_nas .. "/".. probe, "_restart")
      
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Finished syncing configurations. Secondary NAS "..secondary_nas.." restarted.", output_location)
   else
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Failed to retreive config from primary NAS /" .. host.domain .. "/" .. host.hubname .. "/" .. host.robotname, output_location)
   end
   
end

main()